##  ------------------------------------------------------------------------
##    $Id: Dn.pm,v 1.1 2003/01/20 16:48:18 joko Exp $
##  ------------------------------------------------------------------------
##    $Log: Dn.pm,v $
##    Revision 1.1  2003/01/20 16:48:18  joko
##    + initial check-in: refactored from code in Torus::Driver::ldap - helper-module for Data::Storage::Handler::NetLDAP
##
##  ------------------------------------------------------------------------


package Data::Identifier::Dn;

use strict;
use warnings;

use base qw( DesignPattern::Object );


use Data::Transform::Deep qw( deep_copy );

# get logger instance
my $logger = Log::Dispatch::Config->instance;


sub asString {
  my $self = shift;
  
  my $basedn = $self->{base};
  my $type = $self->{type};
  my $ou_address = $self->{name};

  #print "creating deep ou (basedn='$basedn')", "\n" if $DEBUGLEVEL > 3;
  $logger->debug( __PACKAGE__ . "->asString( basedn='$basedn', type='$type', ou_address='$ou_address' )");

#print "basedn: $basedn", "\n";
#print "type: $type", "\n";
#print "ou_address: $ou_address", "\n";

#print "ref: ", ref $ou_address, "\n";

  my @parts = split(/\./, $ou_address);
  
  my $dn = '';
  foreach (reverse @parts) {
    $dn .= "$type=$_, ";
  }
  $dn .= $basedn;

  $logger->debug( __PACKAGE__ . "->asString: dn '$dn'" );

  return $dn;
}

sub getClassName {
  my $self = shift;
  $self->_resolveClassName();
  return $self->{className};
}

sub getType {
  my $self = shift;
  return $self->{type};
}

sub _resolveClassName {
  my $self = shift;
  $self->{type} ||= '';
  # resolve classname from type
  if ($self->{type} eq 'ou') {
    $self->{className} = 'organizationalUnit';
  }
}

sub getContent {
  my $self = shift;
  #$self->_resolveClassName();
  #my $content = "name='$self->{name}', base='$self->{base}', type='$self->{type}', className='$self->{className}'";
  my $content = "name='$self->{name}', base='$self->{base}', type='$self->{type}'";
  return $content;
}

sub getAllRelativeDNs {
  my $self = shift;
  my $dn = $self->asString();
  
  $dn =~ s/$self->{base}//;
  
  my @DNs_work = split(/,+\s+/, $dn);

  my @DNs;
  my @DNs_relative;
  foreach (reverse @DNs_work) {
    
    #print "elem: '", $_, "'\n";
    my $dn_work = $_;
    unshift @DNs_relative, $dn_work;
    
    my $name = $dn_work;
    $name =~ s/^.+?=//;
    
    push @DNs, { 
      full => join(", ", @DNs_relative, $self->{base}),
      relative => deep_copy(\@DNs_relative),
      current => $dn_work,
      name => $name,
    };
  }

  return \@DNs;
  
}

1;
