##  ------------------------------------------------------------------------
##    $Id: MAPI.pm,v 1.1 2003/01/20 16:43:58 joko Exp $
##  ------------------------------------------------------------------------
##    $Log: MAPI.pm,v $
##    Revision 1.1  2003/01/20 16:43:58  joko
##    + initial check-in: refactored from code in Torus::Driver::mapi
##
##  ------------------------------------------------------------------------


package Data::Storage::Handler::MAPI;

use strict;
use warnings;

use base qw(
  Data::Storage::Handler
  Data::Storage::Handler::Abstract
  DesignPattern::Bridge
);


use Data::Dumper;

# we need OLE to access Outlook's Contacts via MAPI
#use Win32::OLE::Strict;
use Win32::OLE;
use Win32::OLE::NLS qw(:LOCALE :DATE);

use Data::Storage::Handler::File qw( s2f );


# get logger instance
my $logger = Log::Dispatch::Config->instance;

my $logfile_errors = '../log/mapi_errors.log';

# debugging?
my $DEBUGLEVEL;
my $DEBUG;


# main vars
my $map;
my $map_ldap;

#my $self->{_COREHANDLE};
my $bool_mailerAlreadyRunning;

sub debugLevel {
  my $level_compare = shift;
  return 1 if ($DEBUGLEVEL > $level_compare);
}



sub getMetaInfo {
  my $self = shift;
  $logger->debug( __PACKAGE__ . "->getMetaInfo()"  );
  return {
    'disconnectMethod' => 'disconnect',
  };
}

sub _init {
  my $self = shift;
  $self->load("Constants");
  $self->load("Extensions");
  #print Dumper($self);
  #$DEBUGLEVEL = $self->{config}->get("debug_level");
  #$DEBUG = ($self->{config}->get("debug_level") > 0);
}

sub connect {

  my $self = shift;

#print Dumper($self);

  $logger->info( __PACKAGE__ . ": creating mailer-object");
  $self->{_COREHANDLE} = Win32::OLE->new('Outlook.Application') or die "oops - could not instantiate Outlook.Application";
  
  #$self->{_COREHANDLE}->Uninitialize();
  #sleep 2;
  
  #$self->{_COREHANDLE}->Initialize();
  
  $bool_mailerAlreadyRunning = $self->mailerCheckRunning();
  
  if ($bool_mailerAlreadyRunning) {
      $logger->info( __PACKAGE__ . ": *not* logging on, using running mailer");
  } else {
      if ($self->{locator}->{showProfileChooser}) {
          $logger->info( __PACKAGE__ . ": logging in (using Profile-Chooser), this may take some seconds!");
          $self->{_COREHANDLE}->Session->Logon ('', '', 1);
      } else {
          $logger->info( __PACKAGE__ . ": logging in (auto-selecting profile " . $self->{locator}->{ProfileName} . "), this may take some seconds!");
          $self->{_COREHANDLE}->Session->Logon($self->{locator}->{ProfileName}, $self->{locator}->{ProfilePass}, 0);
      }
  }

  $self->{locator}->{status}->{connected} = 1;

  return 1;

}

sub _getConnectionInfo {
  my $self = shift;
  my $dsn = shift;
  $dsn =~ m/host=(.+?);binddn=(.+?);pass=(.+?)$/;
  my $args = {
    host => $1,
    binddn => $2,
    pass => $3,
  };
  $args->{binddn} =~ s/'//g if $args->{binddn};
  return $args;
}

sub existsChildNode {
  my $self = shift;
  my $nodeName = shift;

  $logger->debug( __PACKAGE__ . "->existsChildNode( nodeName='$nodeName' )" );

  if (my $folder = $self->getContactFolder($nodeName)) {
    #print "name: ", $folder->Name, "\n";
    return 1;
  } else {
    $logger->debug( __PACKAGE__ . "->existsChildNode: child-node doesn't exist!" );
    return;
  }

  return 1;

}

sub disconnect {
  my $self = shift;

  $logger->info( __PACKAGE__ . ": logging off");
  $self->{_COREHANDLE}->Session->Logoff;
  
  if ($bool_mailerAlreadyRunning || $self->{config}->get("mapi_neverCloseOutlook")) {
      $logger->info( __PACKAGE__ . ": *not* quitting running mailer!");
  } else {
      $logger->info( __PACKAGE__ . ": closing active mail-explorer");
      $self->{_COREHANDLE}->ActiveExplorer->Close;
      $logger->info( __PACKAGE__ . ": quitting mailer");
      $self->{_COREHANDLE}->Quit;
  }

  #$self->{_COREHANDLE}->Uninitialize();

  $logger->info( __PACKAGE__ . ": destroying mailer-object");
  #undef $self->{_COREHANDLE};

}

sub _ldapError {
  my $message = shift;
#print Dumper($message);
  my $textMessage = "LDAP-ERROR: " . $message->error . " - code: " . $message->code;
  $textMessage .= " - dn: " . $message->{matchedDN} if $message->{matchedDN};
  #print "msg: '$textMessage'", "\n";
  #a2f($logfile_errors, $textMessage) if !$TRACELEVEL;
  $logger->error($textMessage);
  return $message;
}

sub getCOREHANDLE {
  my $self = shift;
  return $self->{_COREHANDLE};
}

sub createChildNode {
  my $self = shift;
  my $nodeName = shift;
print "createChildNode: $nodeName", "\n";
  my $nodeDn = Data::Identifier::Dn->new(base => $self->{locator}->{basedn}, type => 'ou', name => $nodeName);
  return $self->createDn($nodeDn);
}

sub getListFiltered {
  my $self = shift;
  my $nodename = shift;
  return $self->getListUnfiltered($nodename);
}


my @list;
sub cb_recieveContact {
  my $contact = shift;
  #print Dumper($contact);
  push @list, $contact;
}

sub getListUnfiltered {
  my $self = shift;
  my $nodeName = shift;
  @list = ();
  $logger->debug( __PACKAGE__ . "->getListUnfiltered( nodeName => '" . $nodeName . "' )" );

  my $folder = $self->getContactFolder($nodeName);
  $self->readMapiFolder($folder, \&cb_recieveContact);

#print Dumper(\@list);

  return \@list;

=pod
  # get list of rows from rdbms by table name
  my $result = $self->sendCommand("SELECT * FROM $nodename");
  while ( my $row = $result->getNextEntry() ) {
    push @list, $row;
  }
  return \@list;
=cut

}

1;
