##  ------------------------------------------------------------------------
##    $Id: Extensions.pm,v 1.1 2003/01/20 16:44:40 joko Exp $
##  ------------------------------------------------------------------------
##    $Log: Extensions.pm,v $
##    Revision 1.1  2003/01/20 16:44:40  joko
##    + initial check-in: refactored from code in Torus::Driver::mapi
##
##  ------------------------------------------------------------------------


package Data::Storage::Handler::MAPI::Extensions;

use strict;
use warnings;

use mixin::with qw( Data::Storage::Handler::MAPI );


use Data::Dumper;

# get logger instance
my $logger = Log::Dispatch::Config->instance;


sub mailerCheckRunning {
  my $self = shift;
    my $tmp;
    # error-handling
    #$tmp = $self->{_COREHANDLE}->ActiveExplorer->WindowState;
    #If Err.Number = 0 Then mailerCheckRunning = True
    #On Error GoTo 0
    return 0;
}

# TODO: add functionality to address arbitrary deep folders anywhere in outlook-scope
# TODO: move 'test' to configuration-scope
sub getContactFolder {
  my $self = shift;
  my $foldername = shift;

  $logger->info( __PACKAGE__ . "->getContactFolder: $foldername");

  my $folder;
  if ($foldername) {
    
    # check if it's a expression describing a hierarchical reference
    if (my @address = split(/\./, $foldername)) {
      #print "trying to find folder: $foldername", "\n";
      return $self->findFolderByAddress(\@address);
    }
    
    my $rootFolder = $self->{_COREHANDLE}->Session->Folders->GetFirst();
    my $folderlist = $rootFolder->Folders;
print Dumper($folderlist);
    for (my $i = 1; $i <= $folderlist->Count; $i++) {
      my $tmpFolder = $folderlist->Item($i);
      #print "folder: ", $tmpFolder->Name, "\n";
      my $name = $tmpFolder->Name;
      if ($name && $name eq $foldername) {
        $folder = $tmpFolder;
        last;
      }
    }
    #exit;
  } else {
    $folder = $self->{_COREHANDLE}->Session->GetDefaultFolder($self->{constants}->{olFolderContacts});
  }
  return $folder;
}

sub findFolderByAddress {
  my $self = shift;
  my $folderAddress = shift;
  my $rootFolder = $self->{_COREHANDLE}->Session->Folders->GetFirst();
  while (my $levelAddress = shift @$folderAddress) {
    $logger->debug( __PACKAGE__ . "->findFolderByAddress: diving into '$levelAddress'.");
    $rootFolder = $self->findFolder($rootFolder, $levelAddress);
  }
  #$self->{_cache}->{folders}->{$folderAddress} = $rootFolder;
  return $rootFolder;
}

sub findFolder {
  my $self = shift;
  my $parentFolder = shift;
  my $findFolderName = shift;

  $logger->debug( __PACKAGE__ . "->findFolder: $findFolderName");

  my $folderlist = $parentFolder->Folders;
  #print Dumper($folderlist);
  for (my $i = 1; $i <= $folderlist->Count; $i++) {
    my $tmpFolder = $folderlist->Item($i);
#print "folder: ", $tmpFolder->Name, "\n";
    my $name = $tmpFolder->Name;
    if ($name && $name eq $findFolderName) {
      return $tmpFolder;
    }
  }
}

sub readMapiFolder {
  my $self = shift;
    my $myFolder = shift;
    $logger->info( __PACKAGE__ . "->readMapiFolder: " . $myFolder->Name);
    #my $sourceFolderName = shift;
    my $callback = shift;
    my $options = shift;
    my $maxCount = $myFolder->Items->Count;
    $logger->info( __PACKAGE__ . "->readMapiFolder: Entries=$maxCount");
    for (my $i = 1; $i <= $maxCount; $i++) {
      #print "Index: $i", "\n";
      my $contactItem = $myFolder->Items->Item($i);
      #print $contactItem->{NickName}, "\n";
      my $item_raw = $self->readContactItem($contactItem);
#print Dumper($item_raw);
      undef $contactItem;
      if ($options->{mapidump}) {
        s2f('../log/mapi_attributes', join("\n", sort keys %$item_raw));
      }
      #&$callback($sourceFolderName, $item_raw);
      &$callback($item_raw);
      print ".";
    }
    undef $myFolder;
}

sub readContactItem {
  
  my $self = shift;

  # we get a real OLE MAPI object here, actually an entry from a MAPI ContactFolder
  my $mapiContactObject = shift;

#print Dumper($mapiContactObject);
  
  # build a list of all MAPI field names
  my @mapiFieldNames = sort keys %{$mapiContactObject};
  
  # OLE -> hash - conversion
  # hash to hold all attributes of the MAPI object
  my $contactItem;

    foreach my $mapi_fieldname (@mapiFieldNames) {
      
      #print "mapi-field: $mapi_fieldname", "\n";
      
      # retrieve value of attribute
      my $value = $mapiContactObject->{$mapi_fieldname};
      #my $value;
      #eval('$value = $mapiContactObject->{$mapi_fieldname};');
      
      # skip empty fields
      next if !$value;

      # patches to values
        # bring all date- and time-fields to unified format
        if ($mapi_fieldname eq 'Anniversary' || $mapi_fieldname eq 'Birthday' || $mapi_fieldname eq 'CreationTime') {
          $value = $value->Date('yyyy-MM-dd') . ' ' . $value->Time('HH:mm');
        }
        if ($mapi_fieldname eq 'UserProperties') {
          # ... do more stuff like this here ...
          #my %hash = %{$value};
          #print "====: ", keys %hash, "\n";
          #print Dumper(%hash);
        }
      
      # TODO: convert Win32::OLE - Variants to perlable variables
      
      # filter any (still existing) real Win32::OLE - variables from object, they would make problems in later processing
      next if ($value =~ m/^Win32::OLE/);
  
      # add attribute from object to hash
       $contactItem->{$mapi_fieldname} = $value;
      
    }

=pod
  if ( debugLevel(3) ) {
    print Dumper($contactItem);
  }
=cut
  
  return $contactItem;
  
}

1;
