############################################
#
#  $Id: Tangram.pm,v 1.25 2003/01/19 02:30:05 joko Exp $
#
#  $Log: Tangram.pm,v $
#  Revision 1.25  2003/01/19 02:30:05  joko
#  + fix: modified call to '_initSchema'
#
#  Revision 1.24  2002/12/22 14:13:01  joko
#  + sub dropDb
#
#  Revision 1.23  2002/12/19 16:31:53  joko
#  +- renamed sub to 'rebuildDb'
#
#  Revision 1.22  2002/12/18 22:28:16  jonen
#  + added extended logging at 'getObjectByGuid()'
#
#  Revision 1.21  2002/12/16 22:20:49  jonen
#  + fixed bug at 'getObjectByGuid()'
#
#  Revision 1.20  2002/12/16 20:49:17  jonen
#  + added sub 'getObjectByGuid()'
#  + added functionality to use 'getObjectByGuid' at 'getObjectAsHash()'
#
#  Revision 1.19  2002/12/16 06:46:09  joko
#  + attempt to introduce a generic '_patchSchema' - cancelled!
#
#  Revision 1.18  2002/12/13 21:48:07  joko
#  + fix to 'sub sendQuery'
#
#  Revision 1.17  2002/12/12 02:51:09  joko
#  + cosmetics
#
#  Revision 1.16  2002/12/11 06:54:10  joko
#  + fix: encapsulated object-loading inside an 'eval'
#
#  Revision 1.15  2002/12/05 13:55:21  joko
#  + now utilizing 'object2hash' instead of 'var_deref'
#  + played around with having fresh-objects - no progress....
#
#  Revision 1.14  2002/12/05 09:40:30  jonen
#  + added option->{destroy} at getObject for unloading all instance
#
#  Revision 1.13  2002/12/05 07:59:04  joko
#  + now using Tie::SecureHash as a base for the COREHANDLE
#  + former public COREHANDLE becomes private _COREHANDLE now
#  + sub getCOREHANDLE
#
#  Revision 1.12  2002/12/04 11:34:49  joko
#  - $schema_tangram doesn't have to be in class?
#
#  Revision 1.11  2002/12/04 08:54:08  jonen
#  + untested bugfix: undef($object) after transform to hash at getObjectAsHash
#
#  Revision 1.10  2002/12/03 15:53:23  joko
#  + small bugfix regarding object hierarchy
#
#  Revision 1.9  2002/12/03 05:29:40  joko
#  + sub getObject
#  + sub getObjectAsHash
#
#  Revision 1.8  2002/12/01 22:25:51  joko
#  + now utilizing metadata from storage locator when connecting to DBI in "raw"-mode
#
#  Revision 1.7  2002/12/01 04:46:19  joko
#  + sub eraseAll
#
#  Revision 1.6  2002/11/29 05:02:30  joko
#  - sub getNewPerlObjectByPkgName (moved to libp.pm)
#  + sub getMetaInfo
#  - sub existsChildNode (moved to Abstract.pm)
#  + sub getListUnfiltered
#  + sub getListFiltered
#  + sub createCursor
#  + sub createSet
#  + sub sendQuery
#
#  Revision 1.5  2002/11/17 06:35:18  joko
#  + locator metadata can now be reached via ->{locator}
#  - getChildNodes is now wrapped via COREHANDLE
#
#  Revision 1.4  2002/10/25 11:44:44  joko
#  + sub _initSchema
#  + sub existsChildNode
#  + sub testIntegrity
#  + sub rebuildDbAndSchema
#
#  Revision 1.3  2002/10/17 03:56:55  joko
#  + bugfix: trapped eval error
#
#  Revision 1.2  2002/10/17 00:10:05  joko
#  + removed dependency from tsobj.pm, schema is now independent
#  + sub getNewPerlObjectByPkgName
#  + sub deploySchema
#  + sub retreatSchema
#
#  Revision 1.1  2002/10/10 03:44:07  cvsjoko
#  + new
#
############################################


package Data::Storage::Handler::Tangram;

use strict;
use warnings;

use base ("Data::Storage::Handler");
use base ("Data::Storage::Handler::Abstract");

use Tangram;
use Data::Dumper;
use libp qw( getNewPerlObjectByPkgName );
use Data::Storage::Result::Tangram;
use Data::Compare::Struct qw( isEmpty );
use Data::Transform::Deep qw( object2hash );
use Data::Transform::Encode qw( var2utf8 );


# get logger instance
my $logger = Log::Dispatch::Config->instance;


# this holds the complete instantiated schema from tangram
my $schema_tangram;

sub getMetaInfo {
  my $self = shift;
  $logger->debug( __PACKAGE__ . "->getMetaInfo()"  );
  return {
    'disconnectMethod' => 'disconnect',
  };
}

sub _initSchema {
  my $self = shift;
  $logger->debug( __PACKAGE__ . "->_initSchema()" );
  #if (!$schema_tangram) {
    my $obj = getNewPerlObjectByPkgName($self->{locator}->{schema}, { EXPORT_OBJECTS => $self->{locator}->{classnames}, want_transactions => $self->{locator}->{want_transactions} } );
    $schema_tangram = $obj->getSchema();
  #}
  if (!$schema_tangram) {
    $logger->error( __PACKAGE__ . "->_initSchema: No Schema available for $self->{schema}" );
    return 0;
  }
  #$self->_patchSchema();
  return 1;
}

sub _patchSchema {
  my $self = shift;
  foreach (keys %{$schema_tangram->{classes}}) {
    next if $schema_tangram->{classes}->{$_}->{abstract};
    #next if ($_ ne 'TsBankAccount');
    #$_ ne 'AbstractAccount' && 
    print "class: $_", "\n";
#print Dumper($schema_tangram->{classes}->{$_});
    # create new string property named 'guid'
    my $tstring = Tangram::String->new();
    $tstring->{name} = $tstring->{col} = 'guid';
    # inject property into schema
    #$schema_tangram->{classes}->{$_}->{root}->{SPECS}->[0]->{fields}->{string}->{$tstring->{name}} = $tstring;
    print Dumper($schema_tangram->{classes}->{$_}->{root}->{SPECS}->[0]->{fields});
  }
}

sub connect {

    my $self = shift;
    
    my $dsn = shift;

#print Dumper($self);
#exit;

    # TODO: re-enable
    $dsn ||= $self->{locator}->{dbi}->{dsn};
    $logger->debug( __PACKAGE__ . "->connect( dsn $dsn )" );
    
    #my $storage = Tangram::Relational->connect( $schema, $dsn );
    #my $storage = Tangram::mysql->connect( $schema, $dsn );
    #$storage = Tangram::Relational->connect( Project->schema, $dsn );
    
#    if (!testDsn($dsn)) {
#      croak("Database at \"$dsn\" is not available");
#      return;
#    }

    #return unless $self->_initSchema();
    $self->_initSchema();

    # create the main tangram storage object
    #$self->{COREHANDLE} = Tangram::Relational->connect( $schema, $dsn );
    $self->{_COREHANDLE} = Tangram::Relational->connect( $schema_tangram, $dsn );

#print "connect", "\n";
#my $core = $self->{_COREHANDLE};
#print Dumper($core);
  
    # some attempts for configuring the wrapped underlying dbi.....
    #$self->{STORAGEHANDLE_UNDERLYING} = $self->getUnderlyingStorage();
    #$self->{STORAGEHANDLE_UNDERLYING}->_configureCOREHANDLE();
    #$self->_configureUnderlyingStorage;
    
    # ..... encapsulation wins!
    $self->configureCOREHANDLE();

  $self->{locator}->{status}->{connected} = 1;

  return 1;

}

sub getChildNodes {

  my $self = shift;
  my @nodes;

  $logger->debug( __PACKAGE__ . "->getChildNodes()" );

  # create new DBI - Data::Storage - object from already connected DBI::db - handle inside the current COREHANDLE
  #my $loc = new Data::Storage::Locator( type => "DBI", dbi => { db => $self->{COREHANDLE}->{db} });
  #my $loc = new Data::Storage::Locator( type => "DBI", COREHANDLE => $self->{COREHANDLE}->{db} );
  
  # todo: should we retrieve information from the schema here
  # rather than poorly getting table names from underlying dbi?
  my $storage = $self->_getSubLayerHandle();
  @nodes = @{$storage->getChildNodes()};
  #$storage->_configureCOREHANDLE();
#print "getchildnodes\n";
#print Dumper($self);
  #if (my $result = $self->sendCommand( 'SHOW TABLES;' ) ) {
  
  # TODO: REVIEW
  #$storage->disconnect();
  
  $self->{meta}->{childnodes} = \@nodes;
  
  return \@nodes;

}


sub testIntegrity {

  my $self = shift;

  $logger->debug( __PACKAGE__ . "->testIntegrity()" );

  # 1st test: are there tables?
  if (!$self->getChildNodes()) {
    $logger->warning( __PACKAGE__ . "->testIntegrity no childnodes exist" );
    return;
  }

  # 2nd test: is there a table named "Tangram"?
  if (!$self->existsChildNode("Tangram")) {
    $logger->warning( __PACKAGE__ . "->testIntegrity childnode \"Tangram\" doesn't exist" );
    return;
  }

  $self->{locator}->{status}->{integrity} = 1;
  return 1;

}


sub _getSubLayerHandle {
  
  my $self = shift;
  
  $logger->debug( __PACKAGE__ . "->_getSubLayerHandle()" );

  #print Dumper($self);

  # hack, make more generic!
  if (!$self->{dataStorageLayer}) {
    $logger->debug( __PACKAGE__ . "->_getSubLayerHandle() creating new dataStorageLayer" );
    #my $loc = Data::Storage::Locator->new( type => "DBI", dbi => $self->{dbi}, COREHANDLE => $self->{COREHANDLE}->{db} );
    my $loc = Data::Storage::Locator->new( { type => "DBI", dbi => $self->{locator}->{dbi} } );
    $self->{dataStorageLayer} = Data::Storage->new( $loc, { protected => 1 } );
    #$self->{STORAGE_UNDER_THE_HOOD}->{STORAGEHANDLE}->_configureCOREHANDLE();
    #$self->{STORAGE_UNDER_THE_HOOD}->_configureCOREHANDLE();
  }
  
  #print Dumper($self->{STORAGE_UNDER_THE_HOOD});
  
  return $self->{dataStorageLayer};
  
}

sub _configureUnderlyingStorage {
  
  my $self = shift;

  $logger->debug( __PACKAGE__ . "->_configureUnderlyingStorage" );

  $self->_configureCOREHANDLE_DBI();
  return;
  
  foreach my $key (keys %{$self->{dbi}}) {
    my $val = $self->{dbi}->{$key};
    print "entry: $key; $val", "\n";
    $self->{_COREHANDLE}->{db}->{$key} = $val;
  }
  #print Dumper($self->{COREHANDLE}->{db});
}


sub configureCOREHANDLE {

  my $self = shift;

  $logger->debug( __PACKAGE__ . "->configureCOREHANDLE" );

  #my $subLayer = $self->_getSubLayerHandle();

  # apply configured modifications
    if (exists $self->{dbi}->{trace_level} && exists $self->{dbi}->{trace_file}) {
      $self->{_COREHANDLE}->{db}->trace($self->{dbi}->{trace_level}, $self->{dbi}->{trace_file});
    }
    if (exists $self->{dbi}->{RaiseError}) {
      $self->{_COREHANDLE}->{db}->{RaiseError} = $self->{dbi}->{RaiseError};
    }
    if (exists $self->{dbi}->{PrintError}) {
      $self->{_COREHANDLE}->{db}->{PrintError} = $self->{dbi}->{PrintError};
    }
    if (exists $self->{dbi}->{HandleError}) {
      $self->{_COREHANDLE}->{db}->{HandleError} = $self->{dbi}->{HandleError};
    }

}

sub deploySchema {
  my $self = shift;
  my $args = shift;
  
  my $dsn = $self->{locator}->{dbi}->{dsn};

  $logger->debug( __PACKAGE__ . "->deploySchema( dsn $dsn )" );

  my $ok;
  if ( my $dbh = DBI->connect($dsn, '', '', $self->{locator}->{dbi} ) ) {
    return unless $self->_initSchema();
    $ok = Tangram::Relational->deploy($schema_tangram, $dbh );
    $dbh->disconnect();
  }
  return $ok;
}

sub retreatSchema {

  my $self = shift;
  my $dsn = $self->{locator}->{dbi}->{dsn};

  $logger->debug( __PACKAGE__ . "->retreatSchema( dsn $dsn )" );

  my $ok;
  if ( my $dbh = DBI->connect($dsn, '', '', $self->{locator}->{dbi} ) ) {

    return unless $self->_initSchema();
    
    #use Data::Dumper; print Dumper($self);
    $self->{dataStorageLayer}->removeLogDispatchHandler("Tangram11");
    
    $ok = Tangram::Relational->retreat($schema_tangram, $dbh );

    # answer "$ok=2" means "maybe" for now - we have to patch this to a constant here because...
    # - ... Tangram::Relational->retreat doesn't seem to return a valid status
    # - possible improvement:
    #   - test this by checking for count of tables in database
    #   - problem with this: there may be some left not having been included to the schema
    #   - maybe better: use "->getChildNodes"?
    $ok = 2;

    $dbh->disconnect();

  }
  return $ok;
}

sub rebuildDb {
  my $self = shift;
  $logger->info( __PACKAGE__ . "->rebuildDb()" );
  my @results;

  # sum up results (bool (0/1)) in array
  push @results, $self->retreatSchema();
  push @results, $self->{dataStorageLayer}->dropDb();
  push @results, $self->{dataStorageLayer}->createDb();
  push @results, $self->deploySchema();

  # scan array for "bad ones"
  my $res = 1;
  map {
    $res = 0 if (!$_);
  } @results;
  
  return $res;
}

sub getListUnfiltered {
  my $self = shift;
  my $nodename = shift;
  my @results;
  $logger->debug( __PACKAGE__ . "->getListUnfiltered( nodename => '" . $nodename . "' )" );
  # get set of objects from odbms by object name
  my $object_set = $self->{_COREHANDLE}->remote($nodename);
  @results = $self->{_COREHANDLE}->select($object_set);
  return \@results;
}

sub getListFiltered {
  my $self = shift;

  # redirect to unfiltered mode
  #return $self->getListUnfiltered(@_);

  my $nodename = shift;
  my $filters = shift;
  my @results;
  $logger->debug( __PACKAGE__ . "->getListFiltered( nodename => '" . $nodename . "' )" );

  #print Dumper($filters);
  
  my @tfilters;
  
  foreach my $filter (@$filters) {
  
    # get filter - TODO: for each filter
    #my $filter = $filters->[0];
  
    # build filter
    my $lexpr = $filter->{key};
    #my $op = $filter->{op};
    my $op = '=';
    my $rexpr = $filter->{val};
    my $tight = 100;
    
  #  my $tfilter = Tangram::Filter->new(
  #    expr => "t1.$lexpr $op '$rexpr'",
  #    tight => $tight,
  #    objects => $objects,
  #  );
  
    # HACK: build eval-string (sorry) to get filtered list - please give advice here
    push @tfilters, '$remote->{' . $filter->{key} . '}' . " $filter->{op} '$filter->{val}'";

  }

  my $tfilter = join(' & ', @tfilters);

  # get set of objects from odbms by object name
  my $remote = $self->{_COREHANDLE}->remote($nodename);
  
  # was:
  #@results = $self->{COREHANDLE}->select($object_set, $tfilter);

  # is:
  # HACK: build eval-string (sorry) to get filtered list - please give advice here
  my $evalstring = 'return $self->{_COREHANDLE}->select($remote, ' . $tfilter . ');';
  
  # get filtered list/set
  @results = eval($evalstring);
  die $@ if $@;
  
  return \@results;
}

sub createCursor {
  my $self = shift;
  my $node = shift;
  my $cmdHandle = $self->{_COREHANDLE}->cursor($node);
  my $result = Data::Storage::Result::Tangram->new( RESULTHANDLE => $cmdHandle );
  return $result;
}

sub createSet {
  my $self = shift;
#print "-" x 80, "\n";
#print Dumper(@_);
  my @objects = @_;
  my $rh = Set::Object->new();
  foreach (@objects) {
    if (!isEmpty($_)) {
#print Dumper($_);
      $rh->insert($_);
    }
  }
  #print Dumper($rh->members());
  my $result = Data::Storage::Result::Tangram->new( RESULTHANDLE => $rh );
  return $result;
}

sub sendQuery {
  my $self = shift;
  my $query = shift;
  #my $sql = "SELECT cs FROM $self->{metainfo}->{$descent}->{node} WHERE $self->{metainfo}->{$descent}->{IdentProvider}->{arg}='$self->{entry}->{source}->{ident}';";
  #my $result = $self->{metainfo}->{$descent}->{storage}->sendCommand($sql);

  #print Dumper($query);

  # HACK: special case: querying by id does not translate into a common tangram query
  # just load the object by given id(ent)
  if ($query->{criterias}->[0]->{key} eq 'id' && $query->{criterias}->[0]->{op} eq 'eq') {
    #print "LOAD!!!", "\n";
    #exit;
    #return Set::Object->new( $self->{COREHANDLE}->load($query->{criterias}->[0]->{val}) );
    my $ident = $query->{criterias}->[0]->{val};
#print "load obj", "\n";
    #return $self->createSet() if $ident == 5;
    $self->{_COREHANDLE}->unload($ident);
    my $object = $self->{_COREHANDLE}->load($ident);
#print "get id", "\n";
    my $oid = $self->{_COREHANDLE}->id($object);
#print Dumper($object);
#print "oid: $oid", "\n";
    return $self->createSet($object);
    #return $self->createSet( $self->{COREHANDLE}->load('300090018') );
  }

  die("This should not be reached for now - redirect to \$self->getListFiltered() here!");

  # TODO: do a common tangram query here

  my @crits;
  foreach (@{$query->{criterias}}) {
    my $op = '';
    $op = '=' if lc $_->{op} eq 'eq';
    push @crits, "$_->{key}$op'$_->{val}'";
  }
  my $subnodes = {};
  map { $subnodes->{$_}++ } @{$query->{subnodes}};
  # HACK: this is hardcoded ;(    expand possibilities!
  #my $crit = join(' AND ', @crits);
  #my $sql = hash2Sql($query->{node}, $subnodes, 'SELECT', $crit);
  #return $self->sendCommand($sql);
  #my $h = $self->{COREHANDLE}->remote($query->{node});
  #my $res = $self->{COREHANDLE}->select($h, $h->{);
  return $self->createCursor($query->{node});
}

sub eraseAll {
  my $self = shift;
  my $classname = shift;
  my $remote = $self->{_COREHANDLE}->remote($classname);
  my @objs = $self->{_COREHANDLE}->select($remote);
  $self->{_COREHANDLE}->erase(@objs);
}

sub createDb {
  my $self = shift;
  my $storage = $self->_getSubLayerHandle();
  return $storage->createDb();
}

sub getObject {
  my $self = shift;
  my $oid = shift;
  my $options = shift;

  # TODO: create a deep_unload method (currently _all_ objects are unloaded)
  # unload($oid) will only unload object, not deep object hashes
  $self->{_COREHANDLE}->unload() if ($options->{destroy});

  # TODO: review this
  #if (!$self->{COREHANDLE}) { return; }

  # TODO: review this
  my $object = eval('$self->{_COREHANDLE}->load($oid);');
  print $@, "\n" if $@;

  return $object if $object;
}

sub getObjectByGuid {
  my $self = shift;
  my $guid = shift;
  my $options = shift;
  
  # Guid and Classname is needed
  if(!$guid || !$options->{classname}) {
   $logger->error( __PACKAGE__ . "->getObjectByGuid: No 'guid' OR no Classname in options hash was given but needed!" );
    return;
  }
  
  # TODO: create a deep_unload method (currently _all_ objects are unloaded)
  # unload($oid) will only unload object, not deep object hashes
  $self->{_COREHANDLE}->unload() if ($options->{destroy});

  # search for object with given Classname and Guid
  my $obj_tmp = $self->{_COREHANDLE}->remote($options->{classname});
  my @result = $self->{_COREHANDLE}->select($obj_tmp, $obj_tmp->{guid} eq $guid);
  
  # we searched for global unique identifer of some object,
  # so I think we can trust there would be only one result
  if($result[0]) {
    return $result[0];
  } else {
    $logger->error( __PACKAGE__ . "->getObjectByGuid: No Object with Classname $options->{classname} and GUID $guid found!" );
    return;
  }
  
}

sub getObjectAsHash {
  my $self = shift;
  my $oid = shift;
  my $options = shift;  
  my $obj;
  
  if($options->{guid}) {
    $obj = $self->getObjectByGuid($oid, $options);
  } else {
    $obj = $self->getObject($oid, $options);
  }
  
  # build options (a callback to unload autovivified objects) for 'expand'
  # TODO: use $logger to write to debug here!
  my $cb; # = sub {};
=pod
  if ($options->{destroy}) {
    $options->{cb}->{destroy} = sub {
      print "================ DESTROY", "\n";
      my $object = shift;
      #print Dumper($object);
      $self->{_COREHANDLE}->unload($object);
      #undef($object);
    };
  }
=cut

  my $hash = object2hash($obj, $options);
  #$options->{cb}->{destroy}->($obj);
  #$self->{_COREHANDLE}->unload($obj);
  
  # convert values in hash to utf8 to be ready for (e.g.) encapsulation in XML
  # now done in object2hash
  #var2utf8($hash) if ($options->{utf8});
  
  # old (wrong) attempts to get rid of used instances, if requested
    #$obj->clear_refs;
    #$self->{COREHANDLE}->unload($obj) if($options->{destroy});
    #$obj->DESTROY;
    #undef($obj) if($options->{destroy});
  
  return $hash;
}

sub getSchema {
  return $schema_tangram;
}

sub getCOREHANDLE {
  my $self = shift;
  return $self->{_COREHANDLE};
}

sub dropDb {
  my $self = shift;
  my $storage = $self->_getSubLayerHandle();
  return $storage->dropDb();
}

1;
