############################################
#
#  $Id: Abstract.pm,v 1.2 2002/12/05 07:59:46 joko Exp $
#
#  $Log: Abstract.pm,v $
#  Revision 1.2  2002/12/05 07:59:46  joko
#  + changed comment
#
#  Revision 1.1  2002/11/29 04:55:45  joko
#  + initial check-in
#
#  Revision 1.1  2002/10/10 03:43:53  cvsjoko
#  + new
#
#
############################################

package Data::Storage::Result::Abstract;

use strict;
use warnings;

use Data::Dumper;

# get logger instance
my $logger = Log::Dispatch::Config->instance;

sub new {
  my $invocant = shift;
  my $class = ref($invocant) || $invocant;
  my $self = { type => $invocant, @_ };
  bless $self, $class;
  return $self;
}

sub AUTOLOAD {
  my $self = shift;
  our $AUTOLOAD;
  return if $AUTOLOAD =~ m/::DESTROY$/;

  # find out methodname
  my $methodname = $AUTOLOAD;
  $methodname =~ s/^.*:://;

  # test for RESULTHANDLE
  if (!$self->{RESULTHANDLE}) {
    $logger->error( __PACKAGE__ .  ": " . "RESULTHANDLE is undefined while trying to execute method \"$methodname\"" );
    return;
  }

  # dispatch method-call to CORE-HANDLE
  if ($self->{RESULTHANDLE}->can($methodname) || $self->{RESULTHANDLE}->can("AUTOLOAD")) {
    #$logger->debug( __PACKAGE__ . "->" . $methodname . "(@_)" );
    $self->{RESULTHANDLE}->$methodname(@_);
  }

}

sub _abstract_function {
  my $self = shift;
  my $fName = shift;
  my $class = ref($self);
  $logger->error( __PACKAGE__ . ": function \"$fName\" is an abstract method, please implement it in \"$class\"");
  #exit;
}

# ====================================================
#   CONCRETE METHOD DUMMIES (croaking via "$logger" by calling "_abstract_function")
# ====================================================

  sub getNextEntry {
    my $self = shift;
    $self->_abstract_function('getNextEntry');
  }

  sub getStatus {
    my $self = shift;
    $self->_abstract_function('getStatus');
  }

1;
