#################################
#
#  $Id: Task.pm,v 1.2 2002/10/17 00:10:43 joko Exp $
#
#  $Log: Task.pm,v $
#  Revision 1.2  2002/10/17 00:10:43  joko
#  + bugfix: constructor argument logging
#
#  Revision 1.1  2002/10/10 03:44:21  cvsjoko
#  + new
#
#
#################################

package Data::Transfer::Job;

use strict;
use warnings;

use misc::HashExt;
use Data::Transfer::Sync;

# get logger instance
my $logger = Log::Dispatch::Config->instance;

sub new {
  my $invocant = shift;
  my $class = ref($invocant) || $invocant;
  my $self = { @_ };
  $logger->debug( __PACKAGE__ . "->new( @_ )" );
  return bless $self, $class;
}


sub run {

  my $self = shift;

  my $storage_left = $self->{storages}->{L};
  my $storage_right = $self->{storages}->{R};
  my $tasks = $self->{steps};
  my $tasklist = $self->{steplist};

      # create synchronization object
      $logger->notice("Starting synchronization (push) between \"Object Store (dbi:" . $storage_left->getDbName() . ")\" and \"Relational Database (dbi:" . $storage_right->getDbName() . ")\"");
      my $syncStep = Data::Transfer::SyncStep->new( L => $storage_left, R => $storage_right );
      
      my $stats_sum = SummableHash->new;
      foreach my $taskKey (@{$tasklist}) {
        my $task = $tasks->{$taskKey};
        my $taskname = $taskKey;
        if ($task->{name}) { $taskname = $task->{name}; }
        #print "-" x 60, "\n";
        #sysevent( { usermsg => $task->{caption}, level => LEVEL_NOTIFY }, $taskEvent );
        $logger->info($task->{caption});
        
        #syncSet2Table( $satellite->{mapping}->$taskname );
        # execute synchronization step (ObjectSet -> Table)
        
        #$syncStep->syncSet2Table( $satellite->{mapping}->$taskname );
        
        if ($task->{coderef}) {
          $task->{coderef}( $syncStep );
        } else {
          my $stats_ref = $syncStep->syncSet2Table( $self->{mapping}->$taskname );
          $stats_sum += $stats_ref;
        }

      }

      # uses overloading via "misc::HashExt" here to transparently stringify hash with statistic-counts
      $logger->notice("stats: $stats_sum");

}

1;