##    $Id: Metadata.pm,v 1.1 2003/01/20 16:58:46 joko Exp $
##
##    Copyright (c) 2002  Andreas Motl <andreas.motl@ilo.de>
##
##    See COPYRIGHT section in pod text below for usage and distribution rights.
##
##    ----------------------------------------------------------------------------------------
##    $Log: Metadata.pm,v $
##    Revision 1.1  2003/01/20 16:58:46  joko
##    + initial check-in: here they are....
##
##    Revision 1.1  2003/01/19 01:23:04  joko
##    + new from Data/Transfer/Sync.pm
##
##    ----------------------------------------------------------------------------------------


package Data::Transfer::Sync::Metadata;

use strict;
use warnings;

use mixin::with qw( Data::Transfer::Sync );


# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -   main

# get logger instance
my $logger = Log::Dispatch::Config->instance;

use Data::Dumper;


sub _preCheckOptions {

  my $self = shift;
  my $opts = shift;

#print Dumper($opts);
#exit;

  # the type of the to-be-synced item
  if (!$opts->{source}->{nodeType}) {
    $logger->error( __PACKAGE__ . "->_preCheckOptions failed: Please specify \"source-type\".");
    return;
  }
  # the name of the (container-) node the items are listed in
  if (!$opts->{source}->{nodeName}) {
    $logger->error( __PACKAGE__ . "->_preCheckOptions failed: Please specify \"source-node\".");
    return;
  }

  # a "map"-declaration which module to use for mapping- and/or lookup-purposes
  if (!$opts->{map}) {
    $logger->warning( __PACKAGE__ . "->_preCheckOptions: No mapping supplied - please check key 'map|mappings' in global configuration or specify additional argument '--mapping-module'.");
    return;
  }
  if (!$opts->{map}->{moduleName}) {
    $logger->warning( __PACKAGE__ . "->_preCheckOptions: Currently only perl-modules can provide mappings: Please specify one with '--mapping-module'.");
    return;
  }

  return 1;

}

# TODO: refactor!!! take this list from already established/given metadata
sub _buildMetadataV1 {
  my $self = shift;

  # decompose identifiers for each partner
  foreach ('source', 'target') {
    
    # get/set metadata for further processing

    # Partner and Node (e.g.: "L:Country" or "R:countries.csv")
    if (my $item = $self->{args}->{$_}) {
      my @item = split(':', $item);
      $self->{meta}->{$_}->{dbkey} = $item[0];
      $self->{meta}->{$_}->{node} = $item[1];
    }
    
    # Filter
    if (my $item_filter = $self->{args}->{$_ . '_filter'}) {
      $self->{meta}->{$_}->{filter} = $item_filter;
    }

    # IdentProvider
    if (my $item_ident = $self->{args}->{$_ . '_ident'}) {
      my @item_ident = split(':', $item_ident);
      $self->{meta}->{$_}->{IdentProvider} = { method => $item_ident[0], arg => $item_ident[1] };
    }

#print Dumper($self->{meta});

    # TODO: ChecksumProvider

    # exclude properties/subnodes
    if (my $item_exclude = $self->{args}->{$_ . '_exclude'}) {
      $self->{meta}->{$_}->{subnodes_exclude} = $item_exclude;
    }
    
    # TypeProvider
    if (my $item_type = $self->{args}->{$_ . '_type'}) {
      my @item_type = split(':', $item_type);
      $self->{meta}->{$_}->{TypeProvider} = { method => $item_type[0], arg => $item_type[1] };
    }
    
    # Callbacks - writers (will be triggered _before_ writing to target)
    if (my $item_writers = $self->{args}->{$_ . '_callbacks_write'}) {
      my $descent = $_;     # this is important since the following code inside the map wants to use its own context variables
      map { $self->{meta}->{$descent}->{Callback}->{write}->{$_}++; } @$item_writers;
    }
    
    # Callbacks - readers (will be triggered _after_ reading from source)
    if (my $item_readers = $self->{args}->{$_ . '_callbacks_read'}) {
      my $descent = $_;
      map { $self->{meta}->{$descent}->{Callback}->{read}->{$_}++; } @$item_readers;
    }
    
    # resolve storage objects
    #$self->{$_} = $self->{container}->{storage}->{$self->{meta}->{$_}->{dbkey}};
    # relink references to metainfo
    $self->{meta}->{$_}->{storage} = $self->{container}->{storage}->{$self->{meta}->{$_}->{dbkey}};
    #print "iiiiisprov: ", Dumper($self->{meta}->{$_}->{storage}), "\n";
  }

}


# TODO: refactor (still)!!! take this list from already established/given metadata ....
# .... also internally: passing around and mungling these metadata doesn't make sense anymore.
sub _buildMetadataV2 {
  my $self = shift;

#print Dumper($self->{options});

  # decompose identifiers for each partner
  foreach ('source', 'target') {
    
    # get/set metadata for further processing

    $self->{options}->{$_}->{dbKey} ||= '';
    $self->{options}->{$_}->{nodeType} ||= '';
    $self->{options}->{$_}->{nodeName} ||= '';

    # Partner and Node (e.g.: "L:Country" or "R:countries.csv")
    $self->{meta}->{$_}->{dbKey} = $self->{options}->{$_}->{dbKey};
    $self->{meta}->{$_}->{nodeType} = $self->{options}->{$_}->{nodeType};
    $self->{meta}->{$_}->{nodeName} = $self->{options}->{$_}->{nodeName};
    
    # Filter
    if (my $item_filter = $self->{args}->{$_ . '_filter'}) {
      $self->{meta}->{$_}->{filter} = $item_filter;
    }

    # IdentProvider
    if (my $item_ident = $self->{options}->{$_}->{ident}) {
      my @item_ident = split(':', $item_ident);
      $self->{meta}->{$_}->{IdentProvider} = { method => $item_ident[0], arg => $item_ident[1] };
    }

    # TODO: ChecksumProvider

    # exclude properties/subnodes
    if (my $item_exclude = $self->{options}->{$_}->{exclude}) {
      $self->{meta}->{$_}->{subnodes_exclude} = $item_exclude;
    }
    
    # TypeProvider
    if (my $item_type = $self->{args}->{$_ . '_type'}) {
      my @item_type = split(':', $item_type);
      $self->{meta}->{$_}->{TypeProvider} = { method => $item_type[0], arg => $item_type[1] };
    }
    
    # Callbacks - writers (will be triggered _before_ writing to target)
    if (my $item_writers = $self->{args}->{$_ . '_callbacks_write'}) {
      my $descent = $_;     # this is important since the following code inside the map wants to use its own context variables
      map { $self->{meta}->{$descent}->{Callback}->{write}->{$_}++; } @$item_writers;
    }
    
    # Callbacks - readers (will be triggered _after_ reading from source)
    if (my $item_readers = $self->{args}->{$_ . '_callbacks_read'}) {
      my $descent = $_;
      map { $self->{meta}->{$descent}->{Callback}->{read}->{$_}++; } @$item_readers;
    }
    
    # resolve storage objects
    $self->{meta}->{$_}->{storage} = $self->{options}->{$_}->{storage}->{handle};
  }

  #print Dumper($self->{meta});
  return 1;

}


sub _buildFieldmappingV1 {
  my $self = shift;

  # build mapping
  # incoming: and Array of node map entries (Array or Hash) - e.g.
  #   [ 'source:item_name' => 'target:class_val' ]
  #   { source => 'event->startDateTime', target => 'begindate' }
  foreach (@{$self->{args}->{mapping}}) {
    if (ref $_ eq 'ARRAY') {
      my @entry1 = split(':', $_->[0]);
      my @entry2 = split(':', $_->[1]);
      my $descent = [];
      my $node = [];
      $descent->[0] = $entry1[0];
      $descent->[1] = $entry2[0];
      $node->[0] = $entry1[1];
      $node->[1] = $entry2[1];
      push @{$self->{meta}->{$descent->[0]}->{childnodes}}, $node->[0];
      push @{$self->{meta}->{$descent->[1]}->{childnodes}}, $node->[1];
    } elsif (ref $_ eq 'HASH') {
      foreach my $entry_key (keys %$_) {
        my $entry_val = $_->{$entry_key};
        push @{$self->{meta}->{$entry_key}->{childnodes}}, $entry_val;
      }
    }

  }

  return 1;

}


1;
