##    $Id: StorageInterface.pm,v 1.1 2003/01/20 16:58:46 joko Exp $
##
##    Copyright (c) 2002  Andreas Motl <andreas.motl@ilo.de>
##
##    See COPYRIGHT section in pod text below for usage and distribution rights.
##
##    ----------------------------------------------------------------------------------------
##    $Log: StorageInterface.pm,v $
##    Revision 1.1  2003/01/20 16:58:46  joko
##    + initial check-in: here they are....
##
##    Revision 1.1  2003/01/19 01:23:04  joko
##    + new from Data/Transfer/Sync.pm
##
##    ----------------------------------------------------------------------------------------


package Data::Transfer::Sync::StorageInterface;

use strict;
use warnings;

use mixin::with qw( Data::Transfer::Sync );


# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -   main

# get logger instance
my $logger = Log::Dispatch::Config->instance;

use Data::Dumper;


# this is a shortcut method
# ... let's try to avoid _any_ redundant code in here (ok... - at the cost of method lookups...)
sub _getNodeList {
  my $self = shift;
  my $descent = shift;
  my $filter = shift;
  #$results ||= $self->{source}->getListUnfiltered($self->{meta}->{source}->{node});
  #$results ||= $self->{meta}->{source}->{storage}->getListUnfiltered($self->{meta}->{source}->{node});
  my $list = $self->{meta}->{$descent}->{storage}->getListFiltered($self->{meta}->{$descent}->{nodeName}, $filter);
#print Dumper($list);
  return $list;
}

sub _resolveNodeIdent {
  my $self = shift;
  my $descent = shift;
  
  #print Dumper($self->{node}->{$descent});
  
  # get to the payload
    #my $item = $specifier->{item};
    my $payload = $self->{node}->{$descent}->{payload};

  # resolve method to get to the id of the given item
  # we use global metadata and the given descent for this task
    #my $ident = $self->{$descent}->id($item);
    #my $ident = $self->{meta}->{$descent}->{storage}->id($item);

    my $ident;
    my $provider_method = $self->{meta}->{$descent}->{IdentProvider}->{method};
    my $provider_arg = $self->{meta}->{$descent}->{IdentProvider}->{arg};

    #print "provider_method: $provider_method", "\n";
    #print "provider_arg: $provider_arg", "\n";

    # resolve to ident
    if (lc $provider_method eq 'property') {
      $ident = $payload->{$provider_arg};

    } elsif (lc $provider_method eq 'storage_method') {
      #$ident = $self->{meta}->{$descent}->{storage}->id($item);
      $ident = $self->{meta}->{$descent}->{storage}->$provider_arg($payload);

    } elsif (lc $provider_method eq 'code') {
      $ident = 'ac';
      
    }
    
    $self->{node}->{$descent}->{ident} = $ident;
    
  return 1 if $ident;

}

# TODO:
# this should be split up into...
#   - a "_statNode" (should just touch the node to check for existance)
#   - a "_loadNode" (should load node completely)
#   - maybe additionally a "loadNodeProperty" (may specify properties to load)
#   - introduce $self->{nodecache} for this purpose
# TODO:
#   should we:
#     - not pass ident in here but resolve it via "$descent"?
#     - refactor this and stuff it with additional debug/error message
#       - this = the way the implicit load mechanism works
sub _statloadNode {

  my $self = shift;
  my $descent = shift;
  my $ident = shift;
  my $force = shift;

  # fetch entry to retrieve checksum from
  # was:
  if (!$self->{node}->{$descent} || $force) {
  # is:
  #if (!$self->{node}->{$descent}->{item} || $force) {
    
    if (!$ident) {
      #print "\n", "Attempt to fetch entry implicitely by ident failed: no ident given! This may result in an insert if no write-protection is in the way.", "\n";
      return;
    }
    
    # patch for DBD::CSV
    if ($ident && $ident eq 'Null') {
      return;
    }

#print "yai!", "\n";

    my $query = { 
      node => $self->{meta}->{$descent}->{nodeName}, 
      subnodes => [qw( cs )], 
      criterias => [ 
        { key => $self->{meta}->{$descent}->{IdentProvider}->{arg}, 
           op => 'eq', 
           val => $ident },
      ]
    };

print Dumper($query);

    my $result = $self->{meta}->{$descent}->{storage}->sendQuery($query);

    my $entry = $result->getNextEntry();

#print Dumper($entry);
#print "pers: " . $self->{meta}->{$descent}->{storage}->is_persistent($entry), "\n";
#my $state = $self->{meta}->{$descent}->{storage}->_fetch_object_state($entry, { name => 'TransactionHop' } );
#print Dumper($state);

    my $status = $result->getStatus();

#print Dumper($status);
    
    # TODO: enhance error handling (store inside tc)
    #if (!$row) {
    #  print "\n", "row error", "\n";
    #  next;
    #}

    # these checks run before actually loading payload- and meta-data to node-container
    
      # 1st level - hard error
      if ($status && $status->{err}) {
        $logger->debug( __PACKAGE__ . "->_statloadNode (ident=\"$ident\") failed - hard error (that's ok): $status->{err}" );
        return;
      }
  
      # 2nd level - logical (empty/notfound) error
      if (($status && $status->{empty}) || !$entry) {
        $logger->debug( __PACKAGE__ . "->_statloadNode (ident=\"$ident\") failed - logical error (that's ok)" );
        #print "no entry (logical)", "\n";
        return;
      }

#print Dumper($entry);

    # was:
    # $self->{node}->{$descent}->{ident} = $ident;  
    # is:
    # TODO: re-resolve ident from entry via metadata "IdentProvider" here - like elsewhere
    $self->{node}->{$descent}->{ident} = $ident;
    $self->{node}->{$descent}->{payload} = $entry;

  }
  
  return 1;
  
}


1;
