## ------------------------------------------------------------------------
##  $Id: Logger.pm,v 1.1 2002/12/22 14:17:38 joko Exp $
## ------------------------------------------------------------------------
##  $Log: Logger.pm,v $
##  Revision 1.1  2002/12/22 14:17:38  joko
##  + initial check-in
##
##  Revision 1.1  2002/12/19 01:14:48  cvsjoko
##  + initial check-in
##
## ------------------------------------------------------------------------


package DesignPattern::Object::Logger;

use strict;
use warnings;

use Data::Dumper;

# TODO: review, revise and rewire this!
my $logger = Log::Dispatch::Config->instance;

sub __init {
  my $self = shift;
  my $action = shift;
  my $message = shift;
  #my $options = shift;   # ??? or: $self->{options} already...?
  
  # set passed-in arguments
  $self->{action} = $action;
  $self->{message} = $message;

  # set default arguments
  $self->{classname} = ref $self;
  $self->{caller} = caller;
  
  # set default values
  $self->{action} ||= '';
  $self->{options}->{tag} ||= '';
  $self->{level} ||= 'info';
}

sub __out {
  my $self = shift;
  my $message = shift;
  
  # if message given here, append to message stored inside object
  $self->{message} .= ' - ' . $message if $message;
  
  # TODO: refactor this:
  #  - push '$self->__get_prefix()' to additional log-entry-metadata, don't show it on/in any console/logfile output
  #$logger->log( level => $self->{level}, message => $self->__get_prefix_long() . $message );
  #$logger->log( level => $self->{level}, message => $message );
  #$logger->warning( __PACKAGE__ . "->_skip: '$caller' (mixed into '$classname'): ($options->{tag}) $message.");
  $logger->log( level => $self->{level}, message => $self->__get_prefix_short() . $self->{message} );
}


sub __handle_options {
  my $self = shift;
  #my $options = shift;   # ??? or: $self->{options} already...?
}

sub __get_prefix_long {
  my $self = shift;
  return __PACKAGE__ . "->$self->{action}: '$self->{caller}' (mixed into '$self->{classname}'): ($self->{options}->{tag}) - ";
}

sub __get_prefix_short {
  my $self = shift;
  my $tag_append = '';
  $tag_append = "$self->{options}->{tag} - " if $self->{options}->{tag};
  return $self->{classname} . "->$self->{action}: $tag_append";
}



sub log {
  my $self = shift;
  my $message = shift;
  $self->{level} = shift;    # TODO: implement propagation of this into Log::Dispatch-object '$logger'!
  $self->{options} = shift;   # TODO: implement additional behaviours: e.g. { exit => 1 } would exit after tracedump
  
  $self->__init('log', $message);
  $self->__out();
  
}

sub skip {
  my $self = shift;
  my $message = shift;
  my $condition = shift;
  $self->{level} = shift;    # TODO: implement propagation of this into Log::Dispatch-object '$logger'!
  $self->{options} = shift;   # TODO: implement additional behaviours: e.g. { exit => 1 } would exit after tracedump

  my $classname = ref $self;
  my $caller = caller;

  $self->{options}->{tag} ||= '';

  if ($condition) {
    # V1:
    #$logger->warning( __PACKAGE__ . "->_skip: '$caller' (mixed into '$classname'): ($options->{tag}) $message.");
    # V2:
    $self->__init('skip', $message);
    $self->__out();
    return 1;
  }
}

sub trace {
  my $self = shift;
  my $message = shift;
  my $data = shift;
  my $condition = shift;
  $self->{level} = shift;    # TODO: implement propagation of this into Log::Dispatch-object '$logger'!
  $self->{options} = shift;   # TODO: implement additional behaviours: e.g. { exit => 1 } would exit after tracedump
  
  my $classname = ref $self;
  my $caller = caller;

  $self->{options}->{tag} ||= '';

  # don't do _any_ tracing if disabled
  return if $self->{TRACE_DISABLED};
  
  # overwrite condition with local configured want-TRACE
  $condition ||= $self->{TRACE};
  
  # dump data
    my $result = 0;
    if ($condition) {
      $message .= " - " . Dumper($data);
      
      # V1:
      # $self->__init('trace');
      # $logger->info( __PACKAGE__ . "->trace: '$caller' (mixed into '$classname'): ($options->{tag}) $message.");
      
      # V2:
      $self->__init('trace', $message);
      $self->__out();
      
      $result = 1;
    } else {
      $result = 0;
    }

  # handle additional options
  if ($self->{TRACE_OPTIONS}) {
    exit if $self->{options}->{exit};
  }

  return $result;
}

1;
