## ------------------------------------------------------------------------
##  $Id: Group.pm,v 1.1 2002/12/22 14:16:39 joko Exp $
## ------------------------------------------------------------------------
##  $Log: Group.pm,v $
##  Revision 1.1  2002/12/22 14:16:39  joko
##  + initial check-in
##
## ------------------------------------------------------------------------


package Regexp::Group;

use strict;
use warnings;

use base 'DesignPattern::Object::Logger';


use Data::Dumper;

use Data::Compare::Struct qw( isEmpty );

my @patterns;

# ------------    common perl object constructor    ------------
sub new {
  my $invocant = shift;
  my $class = ref($invocant) || $invocant;
  my @args = ();
  @_ && (@args = @_);
  #$logger->debug( __PACKAGE__ . "->new( @args )" );      # this is not "common"!
  my $self = { @_ };
  bless $self, $class;
  $self->{caller} = caller;
  
  #print Dumper(caller(2));
  #exit;
  
  $self->_init();
  return $self;
}  

sub _init {
  my $self = shift;
  
  # TODO/REVIEW: compile patterns?
  #@patterns = map{ qr/$_/ } @patterns;
  
  # dereference main/raw data (payload)
  $self->{data} = ${$self->{data}};
}

sub scan {
  my $self = shift;
  my $stepkeys = shift;
  my $coderef = shift;
  
  #my $nodecount_default = shift;
  my $nodecount_default = 19;

#print "SCAN", "\n";
#print $self->{pattern}, "\n";
  $self->{result} = [];
  $self->{result_raw} = [];

  # configure tracing
    # respects additional trace-options passed to _trace-method
    $self->{TRACE_OPTIONS} = 1;
    # disables _any_ tracing - even if sub-conditions evaluate to true values
    $self->{TRACE_DISABLED} = 0;

  $self->log('starting to scan');

  #for (1..20) {
  
  my $patterncount = 0;
  my $match_in_loop;
  
  my $steps;
  if ($stepkeys) {
    $steps = $stepkeys;
    $patterncount++;
  } else {
    $steps = $self->{metadata}->{steps};
  }

#print "cb-2: $coderef", "\n";

  foreach my $stepkey_current (@{$steps}) {
    $self->_scan_step($stepkey_current, $coderef);
    $patterncount++;
  }

  #}

  # ... propagate them to processing
  
    # assign result to property of object's current instance
    #push @{$self->{result}}, $result;
  
  #$self->{code}->($self->{caller}, $self);
  
}

sub _scan_step {

  my $self = shift;
  my $stepkey = shift;
  my $callback = shift;

#print "cb-3: $callback", "\n";

    #$self->log("running step '$stepkey'", 'debug');
    $self->log("running step '$stepkey'");

    my $dataref = $self->{data};
    my $metadata = $self->{metadata}->{$stepkey};
    my $pattern = $self->{patterns}->{$stepkey};
    my $coderef = $self->{coderefs}->{$stepkey};
    
    #print Dumper($coderef);
    #print Dumper($self->{data});
    #exit;

    $self->trace("data used", $dataref, 0, undef, { tag => '', exit => 0 });
    $self->trace("metadata used", $metadata, 0, undef, { tag => '', exit => 0 });
    $self->trace("pattern used", $pattern, 0, undef, { tag => '', exit => 0 });
    $self->trace("coderef used", $coderef, 0, undef, { tag => '', exit => 0 });

#print "pattern: $pattern", "\n";
#exit;

    # if metadata variable "noscan" is supplied, don't kick off the regex-engine, just call the coderef
    # TODO: log this event
    if ($metadata->{noscan} && $metadata->{noscan} == 1) {
      $coderef->($self, undef, undef, $callback);
      return;
    }
    
    # dereference data if its still referenced
    my $data = ${$self->{data}};

    #$self->{data} =~ s{
    $data =~ s{

      $pattern       # the pattern itself - as contained in a string - actually gets interpolated here

    }{

      print "." if $self->{verbose};

      $self->log("match in step '$stepkey'");

      # have a clean item to fill in slots
      my $result_item = [];

      # 1. build matched structure from slots - grouping functionality for slots

        # V1 - hand-rolled
        #my $result = [$1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11, $12, $13, $14, $15, $16, $17, $18, $19, $20];
        #print Dumper($result);

        my $count = $metadata->{nodecount};
        
        # TODO: re-renable this!
        #$count ||= $nodecount_default;

        # V2 - iterate!
        # push dummy item to slot '0' to mimic known behaviour of accessing $1, $2, $3, ....
        push @{$result_item}, 'n/a';
        for my $i (1..$count) {
          my $slot = '$' . $i;
          push @{$result_item}, eval($slot);
        }


      # 2. remember built result (raw) in object - grouping functionality for items
        $self->log("pushing raw result");
        push @{$self->{result_raw}}, $result_item;
  
      
      # 4. check/modify further metadata
      # e.g. 'column_positions' means: "this step is a ColumnProvider"
      #print Dumper($metadata);
        if (my $positions = $metadata->{column_positions}) {
          $self->trace('positions', $positions, 0);
  
  #print Dumper($result_item);
          
          # build column-names array from position-metadata
          my @column_names;
          map {
            push @column_names, $result_item->[$_];
          } @$positions;
          
          # store additional metadata
          $metadata->{column_names} = \@column_names;
        }
        
        $self->trace('column-names', $metadata->{column_names}, 0);
    
        
        #return '';
        #$&;
        push(@{$self->{matches}->{parsed_chunks}}, $&);
        #undef;
  
        my $matchpos = pos($data);
        $self->{matches}->{position} = $matchpos;
        push @{$self->{matches}->{positions}}, $matchpos;


      # 3. call coderef to get _processed_ result if given (callback)
        $self->log("calling coderef '$coderef'");
        if ($coderef) {
          my $result_processed = {};
          if ($coderef->($self, $result_item, $result_processed, $callback)) {
            $self->trace("pushing processed result", $result_processed, 0);
            push @{$self->{result}}, $result_processed if $result_processed;
          } else {
            $self->log('processing of container failed', 'warning');
            return;
          }
        } else {
          # TODO: croak 'no coderef';
        }

      # pass back matched content if we are not the ones to delete something (pass-back-if-not-remover)
      # with this mechanism, we can iterate over the same content again and again and don't loose data while doing this
      # without this, multiple scan stepping would not be possible since each step would either remove its
      # matched content - or not: no flexibility.
      # as it seems we need flexibility at any point ;-( in order not to break our necks with code-maintenance ;-)
      # this also can get configured here (this flag is contained in the metadata-part of your filter-declaration)
      $& if !$metadata->{remover};

    }xge;
  

    #$self->{data_rest} = \$data;
    $self->{data} = \$data;

    #$self->trace('matches', $self->getMatches(), 1);
    #$self->trace('matches-positions', $self->getMatchPositions(), 1);

}

sub continue {
  my $self = shift;
  my $scankey = shift;
  #print "CONTINUE!", "\n";
  #$self->{data} = $self->{data_rest};
  return $self->scan($scankey);
}

sub getMatches {
  my $self = shift;
  return $self->{result_raw};
}

sub getResults {
  my $self = shift;
  return $self->{result};
}

sub getDataRest {
  my $self = shift;
  return $self->{data_rest};
}

sub getMatchCount {
  my $self = shift;

  my $matches = $self->getMatches();
  return if isEmpty($matches);
  
  my $count = $#{$matches};
  $count++;
  return $count;
}

sub getMatchPositions {
  my $self = shift;
  if (!$self->getMatchCount()) {
    $self->{matches}->{positions} = [];
  }
  return $self->{matches}->{positions};
}

sub getMatchPosition {
  my $self = shift;
  return $self->{matches}->{position};
}

1;
