##    --------------------------------------------------------------------------------
##    $Id: libdb.pm,v 1.8 2002/12/12 02:48:58 joko Exp $
##    --------------------------------------------------------------------------------
##    $Log: libdb.pm,v $
##    Revision 1.8  2002/12/12 02:48:58  joko
##    + played with fixing disconnectTarget
##
##    Revision 1.7  2002/12/01 22:13:17  joko
##    + minor bugfix?
##
##    Revision 1.6  2002/11/29 04:53:39  joko
##    + hash2Sql now knows about SQL_SELECT
##
##    Revision 1.5  2002/11/17 07:18:38  joko
##    + small modification in hash2sql
##
##    Revision 1.4  2002/10/16 22:36:42  joko
##    + sub testDbServer
##
##    Revision 1.3  2002/07/27 00:28:20  cvsjoko
##    bugfixes
##
##    Revision 1.2  2002/07/20 11:09:58  cvsjoko
##    + bugfixes
##    + dont' print sql-errors
##
##    Revision 1.1  2002/07/19 18:14:03  cvsjoko
##    no message
##
##
##    --------------------------------------------------------------------------------

package libdb;

use strict;
use warnings;

require Exporter;
our @ISA = qw( Exporter );
our @EXPORT_OK = qw( 
                              testDsn hash2Sql 
                              SQL_INSERT SQL_UPDATE
                              connectTarget disconnectTarget sendSql
                              dbNow
                              getDbNameByDsn sqlDbAction createSqlDb dropSqlDb
                              quotesql
                              testDsnForTables testDbServer
                              );


use libp qw( croak );
use DBI;
use Data::Dumper;

use constant SQL_INSERT => 10;
use constant SQL_UPDATE => 11;
use constant SQL_SELECT => 12;

my $dbmeta_ref_cache;

sub testDsn {
  my $dsn = shift;
  my $result;
  if ( my $dbh = DBI->connect($dsn, '', '', {
                                                      PrintError => 0,
                                                      } ) ) {
    $dbh->disconnect();
    return 1;
  }
}

sub testDbServer {
  my $dsn = shift;
  $dsn =~ s/database=(\w+)//;
  
  #print "testDbServer: $dsn", "\n";
  
  my $result;
  if ( my $dbh = DBI->connect($dsn, '', '', {
                                                      PrintError => 0,
                                                      } ) ) {
    $dbh->disconnect();
    return 1;
  }
}


# TODO: handle usage of "$crit" in an abstract way somehow
sub hash2Sql {
  
  my $table = shift;
  my $hash = shift;
  my $mode = shift;
  my $crit = shift;
  
  my $sql;
  $mode = SQL_SELECT if ($mode eq 'SQL_SELECT' || $mode eq 'SELECT');
  $mode = SQL_INSERT if ($mode eq 'SQL_INSERT' || $mode eq 'INSERT');
  $mode = SQL_UPDATE if ($mode eq 'SQL_UPDATE' || $mode eq 'UPDATE');

  if ($mode == SQL_SELECT) {
    $sql = "SELECT #fields# FROM $table WHERE $crit";
  } elsif ($mode == SQL_INSERT) {
    $sql = "INSERT INTO $table (#fields#) VALUES (#values#)";
  } elsif ($mode == SQL_UPDATE) {
    $sql = "UPDATE $table SET #fields-values# WHERE $crit";
  }
  
  my (@fields, @values);
  foreach my $key (keys %{$hash}) {
    push @fields, $key;
    push @values, $hash->{$key};
  }
  # quote each element
  map { if (defined $_) { $_ = "'$_'" } else { $_ = "null" } } @values;
  
  my $fields = join(', ', @fields);
  my $values = join(', ', @values);
  my $fields_values = '';
  my $fc = 0;
  foreach (@fields) {
    $fields_values .= $_ . '=' . $values[$fc] . ', ';
    $fc++;
  }
  $fields_values = substr($fields_values, 0, -2);
  
  $sql =~ s/#fields#/$fields/;
  $sql =~ s/#values#/$values/;
  $sql =~ s/#fields-values#/$fields_values/;
  
  return $sql;
}

sub patch_dbmeta {
  
  my $dbmeta = shift;

  if ($dbmeta =~ m/^dbi:/) {
    $dbmeta = {
      dsn => $dbmeta,
      trace_level => 0,
      trace_file => undef,
    };
  }
  
  return $dbmeta;
  
}

sub connectTarget {
  my $dbmeta = shift;
  croak "please supply a dsn or a \"dbmeta\"-hash" if (!$dbmeta);
  $dbmeta = patch_dbmeta($dbmeta);
  if (!$dbmeta->{connected}) {
    if ($dbmeta->{dbh} = DBI->connect($dbmeta->{dsn}, '', '', { PrintError => 0 } )) {
      $dbmeta->{dbh}->trace($dbmeta->{trace_level}, $dbmeta->{trace_file});
      $dbmeta->{dbh}->{PrintError} = 0;
      $dbmeta->{connected} = 1;
      $dbmeta_ref_cache = $dbmeta;
      return 1;
    }
  }
}

sub disconnectTarget {
  #my $dbmeta = shift;
  #print "disc\n";
  #croak "please supply a \"dbmeta\"-hash" if (!$dbmeta);
  #$dbmeta = patch_dbmeta($dbmeta);
  #$dbmeta->{dbh} && $dbmeta->{dbh}->disconnect();
  my $dbmeta = $dbmeta_ref_cache;
  #print Dumper($dbmeta);
  #$dbmeta->{dbh} && $dbmeta->{dbh}->disconnect();
  $dbmeta->{dbh}->disconnect();
  undef($dbmeta->{dbh});
  undef($dbmeta);
  undef($dbmeta_ref_cache);
}

sub sendSql {
  my $sql = shift;
  my $dbmeta_ref = shift;
  if (!$dbmeta_ref) { 
    $dbmeta_ref = $dbmeta_ref_cache;
  }
  #print "sql: $sql", "\n";
  if (!$dbmeta_ref->{connected}) {
    print "not connected!", "\n";
    return 0;
  }
  if (my $result = $dbmeta_ref->{dbh}->prepare($sql)) {
    #if ($result->execute()) {
      $result->execute();
      return $result;
    #}
  }
}

sub dbNow {
  #my $now_string = strftime "%a %b %e %H:%M:%S %Y", localtime;
  my $now_string = strftime("%Y-%m-%d %H:%M:%S", localtime);
  return $now_string;
}


sub getDbNameByDsn {
  my $dsn = shift;
  $dsn =~ m/database=(.+?);/;
  my $database_name = $1;
  return $database_name;
}

sub sqlDbAction {
  my $dsn = shift;
  $dsn =~ s/database=.+?;//;
  my $sql = shift;
  my $dbmeta = {
    dsn => $dsn,
    trace_level => 1,
    trace_file => 'dbitrace',
  };
  my $bool_ok;
  if (connectTarget($dbmeta)) {
    sendSql($sql);
    #print "state: ", $dbmeta->{dbh}->state, "\n";
    #print "err    ", $dbmeta->{dbh}->err, "\n";
    #$bool_ok = ($dbmeta->{dbh} && $dbmeta->{dbh}->state && !$dbmeta->{dbh}->err);
    $bool_ok = ($dbmeta->{dbh} && !$dbmeta->{dbh}->err);
    disconnectTarget($dbmeta);
  }
  return $bool_ok;
}

sub createSqlDb {
  my $dsn = shift;
  my $dbname = getDbNameByDsn($dsn);
  print "  - creating rdbms-database $dbname ($dsn) ...";
  my $sql;
  $sql = "CREATE DATABASE $dbname;";
  if ( sqlDbAction($dsn, $sql) ) {
    print "ok", "\n";
    return 1;
  } else {
    print "failed", "\n";
    return 0;
  }
}

sub dropSqlDb {
  my $dsn = shift;
  my $dbname = getDbNameByDsn($dsn);
  print "  - dropping database $dbname", "\n";
  my $sql;
  $sql = "DROP DATABASE $dbname;";
  sqlDbAction($dsn, $sql);
}

sub quotesql {
  my $string = shift;
  if ($string) {
    $string =~ s/'/\\'/g;
  }
  return $string;
}

sub testDsnForTables {
  my $dsn = shift;
  connectTarget($dsn);
  my $result = sendSql('SHOW TABLES;');
  my $bool_tablesHere = $result->fetchrow_hashref();
  $result->finish();
  disconnectTarget($dsn);
  return 1 if ($bool_tablesHere);
}

1;
